@@ -49,7 +49,10 @@ printEncData(EncData *e) {
   printf("FileID: ");
   for(i=0; i < e->fileIDLen; i++) {
     ch = e->fileID[i];
-    printf("%x", ch);
+    if(ch < 16)
+      printf("0%x", ch);
+    else
+      printf("%x", ch);
   }
 
   /** Assume u_string and o_string is of length 32. Not safe, but the code
