@@ -28,7 +28,6 @@
 static FILE *wordList = NULL;
 static const char *wordListName;
 static bool wlMore;
-static bool recovery = false;
 static bool (*npw)() = NULL;
 static unsigned int (*spw)(uint8_t *outbuf) = NULL;
 static passwordMethod pwMethod;
@@ -82,7 +81,7 @@ stdchars[] = {"abcdefghijklmnopqrstuvwxyz"
 	      "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	      "0123456789"};
 
-static const uint8_t *charset = stdchars;
+static const uint8_t *charset;
 static unsigned int charsetLen;
 static unsigned int maxPasswordLen;
 static int password[PASSLENGTH];
@@ -91,7 +90,7 @@ static unsigned int
 genSetPassword(uint8_t *outbuf) {
   unsigned int i;
 
-  for (i=0;i<maxPasswordLen && password[i] != -1;i++)
+  for(i=0;password[i] != -1;i++)
     outbuf[i] = charset[password[i]];
 
   return i;
@@ -102,34 +101,58 @@ static bool
 genNextPassword() {
   unsigned int i = 0;
 
+  /** this is pretty simple...
+      Change the current position (i) to the value in the next position of the
+      charset.
+      If we have reached the end of the charset, move the current position to 
+      the next one and return true unless we have reached the last position we
+      want to try.
+  */
   while(++password[i] == (int)charsetLen)
     password[i++] = 0;
  
   return (i != maxPasswordLen);
 }
 
+static bool recovery = false;
+
 static void
 setCharset(const char *cs, const unsigned int minPw, 
 	   const unsigned int maxPw) {
   int i;
-  int minPasswordLen;
+  unsigned int min;
 
-  if(cs)
-    charset = (uint8_t*)cs;
-  charsetLen = strlen((char*)charset);
-  maxPasswordLen = maxPw;
-  minPasswordLen = (int)minPw;
   npw = &genNextPassword;
   spw = &genSetPassword;
 
   if(!recovery) {
+    /** This should already be set if we are loading from a saved state */
+    if(cs)
+      charset = (uint8_t*)cs;
+    else
+      charset = stdchars;
+    charsetLen = strlen((char*)charset);
+
+    /** Make sure that max- and min-password is smaller than 32 */
+    if(maxPw < PASSLENGTH)
+      maxPasswordLen = maxPw;
+    else
+      maxPasswordLen = PASSLENGTH-1;
+    if(minPw < PASSLENGTH)
+      min = minPw;
+    else
+      min = PASSLENGTH-1;
+
+    /** Initialize starting position */
     for(i=0;i<(int)maxPasswordLen;i++) {
-      if (i<minPasswordLen-1)
+      if (i<((int)min)-1)
 	password[i] = charsetLen-1;
       else
 	password[i] = -1;
     }
   }
+  /** Put terminator (-1) at the last position */
+  password[PASSLENGTH-1] = -1;
 }
 
 void
@@ -138,6 +161,7 @@ initPasswords(const passwordMethod pm, FILE *file, const char *wl,
 	      const unsigned int maxPw) {
   if(!recovery)
     pwMethod = pm;
+
   switch(pwMethod) {
   case Generative:
     setCharset(cs, minPw, maxPw);
@@ -146,58 +170,73 @@ initPasswords(const passwordMethod pm, FILE *file, const char *wl,
     setWordList(file, wl);
     break;
   default:
+    /** The programmer is a twit!  */
     break;
   }
 }
 
-void
+/** Common patterns that is shared between pw_loadState and pw_saveState */
+static const char string_PM[] = "\nPM: %d\n";
+static const char string_MPCLC[] = "MaxPWL: %d\nCharset(%d): ";
+
+bool
 pw_loadState(FILE *file, char **wl) {
   int pm;
   unsigned int i, len;
   char *string;
-  fscanf(file, "PM: %d\n", &pm);
+
+  if(fscanf(file, string_PM, &pm) < 1)
+    return false;
   if(pm == Generative) {
-    fscanf(file, "MaxPWL: %d\nCharsetLength: %d\nCharset: ",
-	   &maxPasswordLen,&charsetLen);
+    if(fscanf(file, string_MPCLC, &maxPasswordLen, &charsetLen) < 2)
+      return false;
+
+    /** check for extremely long charsets */
+    if(charsetLen > 256)
+      return false;
+
     string = malloc(sizeof(uint8_t)*charsetLen+1);
     for(i=0;i<charsetLen;i++)
       string[i] = getc(file);
     string[i] = '\0';
     charset = (uint8_t*)string;
+    
+    /** get the linebreak */
     getc(file);
-    fscanf(file, 
-	   "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
-	   "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
-	   &password[ 0],&password[ 1],&password[ 2],&password[ 3],
-	   &password[ 4],&password[ 5],&password[ 6],&password[ 7],
-	   &password[ 8],&password[ 9],&password[10],&password[11],
-	   &password[12],&password[13],&password[14],&password[15],
-	   &password[16],&password[17],&password[18],&password[19],
-	   &password[20],&password[21],&password[22],&password[23],
-	   &password[24],&password[25],&password[26],&password[27],
-	   &password[28],&password[29],&password[30],&password[31]);
+
+    for(i=0;i<PASSLENGTH-1;i++)
+      if(fscanf(file, " %d", &password[i]) < 1) {
+	free(string);
+	return false;
+      }
   }
   else if(pm == Wordlist) {
-    fscanf(file, "Wordlist(%d): ", &len);
+    if(fscanf(file, "Wordlist(%d): ", &len) < 1)
+      return false;
     string = malloc(sizeof(char)*len+1);
-    fscanf(file, "%[^\n]\n", string);
+    if(fscanf(file, "%[^\n]\n", string) < 1) {
+	free(string);
+	return false;
+    }
     string[len] = '\0';
     *wl = string;
     wordListName = string;
   }
   pwMethod = pm;
   recovery = true;
+
+  return true;
 }
 
 void
 pw_saveState(FILE *file) {
   unsigned int i;
-  fprintf(file, "PM: %d\n", pwMethod);
+  fprintf(file, string_PM, pwMethod);
   if(pwMethod == Generative) {
-    fprintf(file, "MaxPWL: %d\nCharsetLength: %d\nCharset: %s\n",
-	    maxPasswordLen,charsetLen,charset);
-    for(i=0;i<PASSLENGTH;i++)
-      fprintf(file,"%d ", password[i]);
+    fprintf(file, string_MPCLC, maxPasswordLen, charsetLen);
+    fprintf(file, "%s\n", charset);
+    for(i=0;i<PASSLENGTH-1;i++)
+      fprintf(file," %d", password[i]);
   }
   else if(pwMethod == Wordlist) {    
     fprintf(file, "Wordlist(%d): %s", strlen(wordListName), wordListName);
