@@ -18,7 +18,6 @@
  * USA.
  */
 
-#include <stdio.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -58,7 +57,7 @@ static unsigned int currPWLen;
 
 /** statistics */
 static unsigned int nrprocessed;
-static clock_t startTime;
+static time_t startTime;
 
 /** pointer to the actual encoding-data from the pdf */
 static const EncData *encdata;
@@ -68,24 +67,23 @@ static bool crackDone;
 static bool knownPassword;
 static bool workWithUser;
 
-
 /** Print out some statistics */
 bool
 printProgress(void) {
-  clock_t currentTime;
+  time_t currentTime;
   char str[33];
 
   if(crackDone)
     return true;
 
-  currentTime = clock();
+  currentTime = time(NULL);
   memcpy(str,currPW,currPWLen);
   str[currPWLen] = '\0';
   printf("Average Speed: %.1f w/s. ",
-	 nrprocessed/(((double)(currentTime-startTime))/CLOCKS_PER_SEC));
+	 nrprocessed/difftime(currentTime,startTime));
   printf("Current Word: '%s'\n",str);
   nrprocessed = 0;
-  startTime = clock();
+  startTime = time(NULL);
   return false;
 }
 
@@ -138,16 +136,24 @@ initEncKeyWorkSpace(const int revision, const bool encMetaData,
   return size+fileIDLen;
 }
 
+#if 0
 /** For debug */
-/**
 static void
-printString(const uint8_t *str, const unsigned int len) {
+printHexString(const uint8_t *str, const unsigned int len) {
   unsigned int i;
   for(i=0;i<len;i++)
     printf("%x ",str[i]);
   printf("\n");
 }
-*/
+
+static void
+printString(const uint8_t *str, const unsigned int len) {
+  unsigned int i;
+  for(i=0;i<len;i++)
+    printf("%d ",str[i]);
+  printf("\n");
+}
+#endif
 
 /** toupper(3) but expanded to handle iso-latin-1 characters */
 static uint8_t
@@ -161,24 +167,44 @@ isolat1ToUpper(const uint8_t b) {
 /** Really stupid permutate that needs to be replaced with a better framwork
     for smart permutations of the current password */
 static bool
-permutate(void) {
-  static bool ret = true;
-  
-  currPW[0] = isolat1ToUpper(currPW[0]);
-  ret = !ret;
-  
+do_permutate(void) {
+  static bool ret = false;
+  uint8_t tmp;
+
+  tmp = isolat1ToUpper(currPW[0]);
+  if(tmp != currPW[0]) {
+    currPW[0] = tmp;
+    ret = !ret;
+  }
+  else
+    ret = false;  
+
   return ret;
 }
 
+/** Dummy-function to use when no permutations are wanted */
+static bool
+no_permutate(void) { return false; }
+
+/** Placeholder for the correct permutation-function to run */
+static bool (*permutate)() = NULL;
+
+/** Prints out the password found */
 static void
 foundPassword(void) {
   char str[33];
 
+  /** The only password that can be found in plaintext is the password pointed
+      to by currPW. If we for example would like to print out the userpassword
+      when we have found the correct ownerpassword we need to recalculate it
+      from this password
+  */
   memcpy(str,currPW,currPWLen);
   str[currPWLen] = '\0';
   printf("found %s-password: '%s'\n", workWithUser?"user":"owner", str);
 }
 
+/** Common handling of the key for all rev3-functions */
 #define RC4_DECRYPT_REV3(n) {			\
     for(i = 19; i >= 0; --i) {			\
       for(j = 0; j < length; ++j)		\
@@ -187,15 +213,21 @@ foundPassword(void) {
     }						\
   }
 
+/** Checks if the rev2-password set up in encKeyWorkSpace is the correct one
+    and return true if it is and false otherwise.
+*/
 static bool
 isUserPasswordRev2(void) {
   uint8_t enckey[16];
 
   md5(encKeyWorkSpace, ekwlen, enckey);
 
-  return rc4Match40b(enckey, encdata->u_string, 32, pad);
+  return rc4Match40b(enckey, encdata->u_string, pad);
 }
 
+/** Checks if the rev3-password set up in encKeyWorkSpace is the correct one
+    and return true if it is and false otherwise.
+*/
 static bool
 isUserPasswordRev3(void) {
   uint8_t test[16], enckey[16], tmpkey[16];
@@ -209,7 +241,7 @@ isUserPasswordRev3(void) {
 
   RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
       
-  /** if partial test succeeds we make a full check to make sure */
+  /** if partial test succeeds we make a full check to be sure */
   if(unlikely(memcmp(test, rev3TestKey, PARTIAL_TEST_SIZE) == 0)) {
     memcpy(test, encdata->u_string, 16);
     RC4_DECRYPT_REV3(16);
@@ -220,6 +252,7 @@ isUserPasswordRev3(void) {
   return false;
 }
 
+/** Common beginning of the main-loop in all the cracking-functions */
 #define BEGIN_CRACK_LOOP() {				\
     currPWLen = setPassword(currPW);			\
     if(unlikely(lpasslength != currPWLen)) {		\
@@ -229,13 +262,13 @@ isUserPasswordRev3(void) {
     }							\
   }
 
-static bool
+bool
 runCrackRev2_o(void) {
   uint8_t enckey[16];
   unsigned int lpasslength;
 
   lpasslength = 0;
-  startTime = clock();
+  startTime = time(NULL);
   while(nextPassword()) {
     BEGIN_CRACK_LOOP();
   
@@ -243,7 +276,8 @@ runCrackRev2_o(void) {
       md5(currPW, 32, enckey);
 
       rc4Decrypt(enckey, encdata->o_string, 32, encKeyWorkSpace);
-      if(isUserPasswordRev2())
+      md5(encKeyWorkSpace, ekwlen, enckey);
+      if(rc4Match40b(enckey, encdata->u_string, pad))
 	return true;
 
       ++nrprocessed;
@@ -252,7 +286,7 @@ runCrackRev2_o(void) {
   return false;
 }
 
-static bool
+bool
 runCrackRev3_o(void) {
   uint8_t test[32], enckey[16], tmpkey[16];
   unsigned int j, length, lpasslength;
@@ -260,7 +294,7 @@ runCrackRev3_o(void) {
 
   length = encdata->length/8;
   lpasslength = 0;
-  startTime = clock();
+  startTime = time(NULL);
   while(nextPassword()) {
     BEGIN_CRACK_LOOP();
 
@@ -282,13 +316,13 @@ runCrackRev3_o(void) {
   return false;
 }
 
-static bool
+bool
 runCrackRev2_of(void) {
   uint8_t enckey[16];
   unsigned int lpasslength;
 
   lpasslength = 0;
-  startTime = clock();
+  startTime = time(NULL);
   while(nextPassword()) {
     BEGIN_CRACK_LOOP();
 
@@ -296,7 +330,7 @@ runCrackRev2_of(void) {
       md5(encKeyWorkSpace, 32, enckey);
 
       /* Algorithm 3.4 reversed */
-      if(rc4Match40b(enckey, encdata->o_string, 32, password_user))
+      if(rc4Match40b(enckey, encdata->o_string, password_user))
 	return true;
 
       ++nrprocessed;
@@ -305,7 +339,7 @@ runCrackRev2_of(void) {
   return false;
 }
 
-static bool
+bool
 runCrackRev3_of(void) {
   uint8_t test[32], enckey[16], tmpkey[16];
   unsigned int j, length, lpasslength;
@@ -313,7 +347,7 @@ runCrackRev3_of(void) {
 
   length = encdata->length/8;
   lpasslength = 0;
-  startTime = clock();
+  startTime = time(NULL);
   while(nextPassword()) {
     BEGIN_CRACK_LOOP();
     
@@ -325,7 +359,7 @@ runCrackRev3_of(void) {
       memcpy(test, encdata->o_string, 32);
       RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
 
-      /** if partial test succeeds we make a full check to make sure */
+      /** if partial test succeeds we make a full check to be sure */
       if(unlikely(memcmp(test, password_user, PARTIAL_TEST_SIZE) == 0)) {
 	memcpy(test, encdata->o_string, 32);
 	RC4_DECRYPT_REV3(32);
@@ -339,7 +373,7 @@ runCrackRev3_of(void) {
   return false;
 }
 
-static bool
+bool
 runCrackRev3(void) {
   uint8_t test[16], enckey[16], tmpkey[16];
   unsigned int j, length, lpasslength;
@@ -347,7 +381,7 @@ runCrackRev3(void) {
 
   length = encdata->length/8;
   lpasslength = 0;
-  startTime = clock();
+  startTime = time(NULL);
   while(nextPassword()) {
     BEGIN_CRACK_LOOP();
 
@@ -360,7 +394,7 @@ runCrackRev3(void) {
      /** Algorithm 3.5 reversed */  
       RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
 
-      /** if partial test succeeds we make a full check to make sure */
+      /** if partial test succeeds we make a full check to be sure */
       if(unlikely(memcmp(test, rev3TestKey, PARTIAL_TEST_SIZE) == 0)) {
 	memcpy(test, encdata->u_string, 16);
 	RC4_DECRYPT_REV3(16);
@@ -375,13 +409,13 @@ runCrackRev3(void) {
 }
 
 
-static bool
+bool
 runCrackRev2(void) {
   uint8_t enckey[16];
   unsigned int lpasslength;
 
   lpasslength = 0;
-  startTime = clock();
+  startTime = time(NULL);
   while(nextPassword()) {
     BEGIN_CRACK_LOOP();
 
@@ -389,7 +423,7 @@ runCrackRev2(void) {
       md5(encKeyWorkSpace, ekwlen, enckey);
 
       /* Algorithm 3.4 reversed */
-      if(rc4Match40b(enckey, encdata->u_string, 32, pad))
+      if(rc4Match40b(enckey, encdata->u_string, pad))
 	return true;
 
       ++nrprocessed;
@@ -398,6 +432,9 @@ runCrackRev2(void) {
   return false;
 }
 
+/** Start cracking and does not stop until it has either been interrupted by
+    a signal or the password either is found or not found.
+*/
 void
 runCrack(void) {
   bool found = false;
@@ -430,19 +467,48 @@ runCrack(void) {
   currPW = NULL;
 }
 
+/** returns the number of processed passwords */
+unsigned int
+getNrProcessed(void) { return nrprocessed; }
+
+/** These are shared variables between loading and initialisation and controls
+    how to do the initialisation. Should not be touched by anything except 
+    loadState and cleanPDFCrack.
+*/
+static bool recovery = false;
+static bool permutation = false;
+
+
+/** cleans up everything as is needed to do a any initPDFCrack-calls after the
+    first one.
+*/
 void
 cleanPDFCrack(void) {
-  if(rev3TestKey)
-    free(rev3TestKey);
-  if(encKeyWorkSpace)
+  if(rev3TestKey) {
+    /** Do a really ugly const to non-const cast but this one time it should
+	be safe
+    */
+    free((uint8_t*)rev3TestKey);
+    rev3TestKey = NULL;
+  }
+  if(encKeyWorkSpace) {
     free(encKeyWorkSpace);
+    encKeyWorkSpace = NULL;
+  }
+  knownPassword = false;
+  recovery = false;
+  permutation = false;
 }
 
+/** initPDFCrack is doing all the initialisations before you are able to call
+    runCrack(). Make sure that you run cleanPDFCrack before you call this 
+    after the first time.
+*/
 bool
 initPDFCrack(const EncData *e, const uint8_t *upw, const bool user,
 	     const char *wl, const passwordMethod pm, FILE *file, 
 	     const char *cs, const unsigned int minPw, 
-	     const unsigned int maxPw) {
+	     const unsigned int maxPw, const bool perm) {
   uint8_t buf[128];
   unsigned int upwlen;
   uint8_t *tmp;
@@ -466,6 +532,9 @@ initPDFCrack(const EncData *e, const uint8_t *upw, const bool user,
     memcpy(encKeyWorkSpace, password_user, 32);
     knownPassword = true;
   }
+  /** Workaround to set password_user when loading state from file */
+  if(recovery) 
+    memcpy(encKeyWorkSpace, password_user, 32);
 
   if(encdata->revision == 2) {
     if(knownPassword) {
@@ -494,76 +563,146 @@ initPDFCrack(const EncData *e, const uint8_t *upw, const bool user,
       knownPassword = isUserPasswordRev3();
     }
   }
+
+  if(perm || permutation)
+    permutate = do_permutate;
+  else
+    permutate = no_permutate;
   
+
   initPasswords(pm, file, wl, cs, minPw, maxPw);
   return true;
 }
 
-void
+/** Some common patterns between the loadState and saveState */
+static const char string_PRVPL[] = 
+  "PDF: %d.%d\nR: %d\nV: %d\nP: %d\nL: %d\n"
+  "MetaData: %d\nFileID(%d):";
+static const char string_FILTER[] = "\nFilter(%d): ";
+static const char string_UUPWP[] = 
+  "\nUser: %d\nUserPw: %d\nPermutate: %d\n";
+
+
+/** Reads from file and tries to set up the state that it contains.
+    Returns true on success and false otherwise.
+    Very little checks for data validitiy is made.
+*/
+bool
 loadState(FILE *file, EncData *e, char **wl, bool *user) {
   unsigned int i;
-  int tmp;
+  int tmp, tmp2, tmp3, len;
+  char c;
+
+  /** Load all the simple values bound to the document */
+  if(fscanf(file,string_PRVPL, &e->version_major, &e->version_minor,
+	    &e->revision, &e->version, &e->permissions, &e->length,
+	    &tmp, &e->fileIDLen) < 8)
+    return false;
+
+  /** bork out if length is insanely high */
+  if(e->fileIDLen > 256)
+    return false;
 
-  /** possible overflow, but I really don't care */
-  e->s_handler = malloc(sizeof(uint8_t)*32);
-  fscanf(file,"PDF: %d.%d\nR: %d\nV: %d\nP: %d\nL: %d\n"
-	 "Filter: %[^\n]\nFileIDLen: %d\nMetaData: %d\nFileID: ",
-	 &e->version_major, &e->version_minor, &e->revision, &e->version,
-	 &e->permissions, &e->length, e->s_handler, &e->fileIDLen, &tmp);
   e->encryptMetaData = (tmp == true);
+
+  /** Load the FileID */
   e->fileID = malloc(sizeof(uint8_t)*e->fileIDLen);
-  e->o_string = malloc(sizeof(uint8_t)*32);
-  e->u_string = malloc(sizeof(uint8_t)*32);
   for(i=0;i<e->fileIDLen;i++) {
-    fscanf(file, "%d ", &tmp);
+    if(fscanf(file, " %d", &tmp) < 1)
+      return false;
     e->fileID[i] = tmp;
   }
-  fscanf(file,"\nO: ");
+
+  /** Load the Security Handler */
+  if(fscanf(file,string_FILTER, &len) < 1)
+    return false;
+
+  /** bork out if length is insanely high or negative */
+  if(len > 256 || len < 0)
+    return false;
+
+  if(len > 0)
+    e->s_handler = malloc((sizeof(uint8_t)*len)+1);
+
+  for(i=0;i<(unsigned int)len;i++) {
+    if(fscanf(file, "%c", &c) < 1)
+      return false;
+    e->s_handler[i] = c;
+  }
+  /** Make sure we null-terminate the string */
+  e->s_handler[i] = '\0';
+
+  /** Load the U- and O-strings */
+  if(fscanf(file, "\nO:") == EOF)
+    return false;
+  e->o_string = malloc(sizeof(uint8_t)*32);
+  e->u_string = malloc(sizeof(uint8_t)*32);
   for(i=0;i<32;i++) {
-    fscanf(file, "%d ", &tmp);
+    if(fscanf(file, " %d", &tmp) < 1)
+      return false;
     e->o_string[i] = tmp;
   }
-  fscanf(file,"\nU: ");
+  if(fscanf(file, "\nU:") == EOF)
+    return false;
   for(i=0;i<32;i++) {
-    fscanf(file, "%d ", &tmp);
+    if(fscanf(file, " %d", &tmp) < 1)
+      return false;
     e->u_string[i] = tmp;
   }
-  fscanf(file, "\nUser: %d", &tmp);
+ 
+  /** Load the simple values bound to the state */
+  if(fscanf(file, string_UUPWP, &tmp, &tmp2, &tmp3) < 3)
+    return false;
   *user = (tmp == true);
-  fscanf(file, "\nUserPw: %d\n", &tmp);
-  knownPassword = (tmp == true);
-  if(knownPassword)
+  knownPassword = (tmp2 == true);
+  permutation = (tmp3 == true);
+
+  /** Load the saved userpassword if that is used */
+  if(knownPassword) {
     for(i=0;i<32;i++) {
-      fscanf(file, "%d ", &tmp);
+      if(fscanf(file, " %d", &tmp) < 1)
+	return false;
       password_user[i] = tmp;
     }
-  fscanf(file, "\n");
-  pw_loadState(file, wl);
+  }
+
+  /** Load the password-specific stuff for the state */
+  if(!pw_loadState(file, wl))
+    return false;
+
+  /** Remember to set the recovery-boolean to make sure the right things 
+      happen in initPDFCrack */
+  recovery = true;
+
+  return true;
 }
 
+/** Saves the current state in the engine to file */
 void
 saveState(FILE *file) {
   unsigned int i;
 
-  fprintf(file, "PDF: %d.%d\nR: %d\nV: %d\nP: %d\nL: %d\n"
-	  "Filter: %s\nFileIDLen: %d\nMetaData: %d\nFileID: ",
+  fprintf(file, string_PRVPL,
 	  encdata->version_major, encdata->version_minor, encdata->revision,
 	  encdata->version, encdata->permissions, encdata->length,
-	  encdata->s_handler,encdata->fileIDLen,(int)encdata->encryptMetaData);
+	  (int)encdata->encryptMetaData, encdata->fileIDLen);
   for(i=0;i<encdata->fileIDLen;i++)
-    fprintf(file, "%d ", encdata->fileID[i]);
-  fprintf(file,"\nO: ");
+    fprintf(file, " %d", encdata->fileID[i]);
+  fprintf(file, string_FILTER, strlen(encdata->s_handler));
+  fprintf(file, "%s", encdata->s_handler);
+  fprintf(file, "\nO:");
   for(i=0;i<32;i++)
-    fprintf(file, "%d ", encdata->o_string[i]);
-  fprintf(file,"\nU: ");
+    fprintf(file, " %d", encdata->o_string[i]);
+  fprintf(file,"\nU:");
   for(i=0;i<32;i++)
-    fprintf(file, "%d ", encdata->u_string[i]);
-  fprintf(file, "\nUser: %d\nUserPw: %d\n",
-	  (int)workWithUser, (int)knownPassword);
+    fprintf(file, " %d", encdata->u_string[i]);
+  fprintf(file, string_UUPWP, (int)workWithUser,
+	  (int)knownPassword, (int)permutation);
   if(knownPassword) {
     for(i=0;i<32;i++)
-      fprintf(file, "%d ", password_user[i]);
+      fprintf(file, " %d", password_user[i]);
   }
-  fprintf(file, "\n");
+
+  /** Save the password-specific stuff */
   pw_saveState(file);
 }
